/**
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 * <p>
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * Antony Hosking, David Whitlock, and Nathaniel Nystrom.  No charge
 * may be made for copies, derivations, or distributions of this
 * material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package EDU.purdue.cs.bloat.file;

import EDU.purdue.cs.bloat.reflect.*;
import java.util.*;
import java.io.*;

/**
 * The Java Virtual Machine Specification allows implementors to 
 * invent their own attributes.  GenericAttribute models attributes 
 * whose name BLOAT does not recognize.
 *
 * @author Nate Nystrom
 *         (<a href="mailto:nystrom@cs.purdue.edu">nystrom@cs.purdue.edu</a>)
 */
public class GenericAttribute extends Attribute
{
  private byte[] data;

  /**
   * Constructor.  Create an attribute from a data stream.
   *
   * @param in
   *        The data stream of the class file.
   * @param nameIndex
   *        The index into the constant pool of the name of the attribute.
   * @param length
   *        The length of the attribute, excluding the header.
   * @exception IOException
   *        If an error occurs while reading.
   */
  public GenericAttribute(DataInputStream in, int nameIndex, int length)
    throws IOException
  {
    super(nameIndex, length);
    data = new byte[length];
    for (int read = 0; read < length;) {
      read += in.read(data, read, length - read);
    }
  }

  /**
   * Write the attribute to a data stream.
   *
   * @param out
   *        The data stream of the class file.
   * @exception IOException
   *        If an error occurs while writing.
   */
  public void writeData(DataOutputStream out) throws IOException
  {
    out.write(data, 0, data.length);
  }

  /**
   * Private constructor used in cloning.
   */
  private GenericAttribute(GenericAttribute other) {
    super(other.nameIndex, other.length);

    this.data = new byte[other.data.length];
    System.arraycopy(other.data, 0, this.data, 0, other.data.length);
  }

  public Object clone() {
    return(new GenericAttribute(this));
  }
}
